#ifndef GLWIDGET_H
#define GLWIDGET_H

#include <QGLWidget>

#include "Common.h"

#include <vector>

#include "GfxObject.h"
#include "Interpolator.h"

class GLWidget : public QGLWidget
{
    Q_OBJECT

public:
    GLWidget(QWidget *parent = 0);
    ~GLWidget();

    QSize minimumSizeHint() const;
    QSize sizeHint() const;

    void addGfxObject( GfxObject* o );

    void incZ(){ _eyeZ += 10; }
    void decZ(){ _eyeZ -= 10; }
    
public slots:
    void animate();
    void setXRotation(int angle);
    void setYRotation(int angle);
    void setZRotation(int angle);
/*
signals:
    void xRotationChanged(int angle);
    void yRotationChanged(int angle);
    void zRotationChanged(int angle);
*/    
protected:
    void initializeGL();
    void paintGL();
    void resizeGL( int width, int height );
    void mousePressEvent( QMouseEvent *event );
    void mouseMoveEvent( QMouseEvent *event );
    void mouseDoubleClickEvent( QMouseEvent *event );
    
private:
    vector<GfxObject*> _objects;
    
    int xRot;
    int yRot;
    int zRot;

    Interpolator xInt;
    Interpolator yInt;

    float _eyeZ;
    float _width;
    float _height;
    
    QPoint _lastPos;

    void gfxOrtho( float ratio, float height, float near, float far );
    void gfxPerspective( float ratio, float height, float near, float far, 
                        float depth );
};

#endif
