#include "GfxFtStringObject.h"

/**
 * Default constructor
 */
GfxFtStringObject::GfxFtStringObject( const char* fontPath )
:_font(fontPath)
,_size(10)
,_depth(1)
{
    _font.CharMap(ft_encoding_unicode);
    _font.FaceSize( _size );
    _font.Depth( _depth );
}

/**
 * Constructor
 */
GfxFtStringObject::GfxFtStringObject( string& s, const char* fontPath )
:_text(s)
,_font(fontPath)
,_size(10)
,_depth(1)
{
    _font.CharMap(ft_encoding_unicode);
    _font.FaceSize( _size );
    _font.Depth( _depth );
}

/**
 * Destructor
 */
GfxFtStringObject::~GfxFtStringObject()
{
}

/**
 * Returns the width of the object.
 */
float GfxFtStringObject::width()
{
    FTBBox bBox = _font.BBox( _text.c_str() );
    return ::abs( bBox.Upper().X() - bBox.Lower().X() );
}

/**
 * Returns the height of the object.
 */
float GfxFtStringObject::height()
{
    FTBBox bBox = _font.BBox( _text.c_str() );
    return ::abs( bBox.Upper().Y() - bBox.Lower().Y() );
}

/**
 * Sets the text.
 */
void GfxFtStringObject::setText( string& t )
{
    _text = t;
}

/**
 * Sets the title.
 */
void GfxFtStringObject::setText( const char* s )
{
    string t(s);
    setText( t );
}


/**
 * Sets the depth.
 */
void GfxFtStringObject::setDepth( float d )
{
    _depth = d;
    _font.Depth( d );
}

/**
 * Sets the font size
 */
void GfxFtStringObject::setFontSize( unsigned int s )
{
    _size = s;
    _font.FaceSize( _size );
}

// @Override
void GfxFtStringObject::drawComponent()
{    
    glTranslatef( 0, 0, _depth );
    
	_font.Render( _text.c_str() );
}
