#ifndef _GFXOBJECT_H_
#define _GFXOBJECT_H_

#include "Common.h"

//#include <QGLWidget>
#include <QVector3D>
#include <QColor>

#include <GL/glut.h>

/**
 * Abstract class for graphics objects.
 */
class GfxObject
{
public:
    GfxObject();
    virtual ~GfxObject();

    void setVisible( bool v ){ _isVisible = v; }
    bool isVisible() { return _isVisible; }

    // positioning
    void translate( float x, float y, float z );
    QVector3D& translate(){ return _translation; }
    
    void rotate( float x, float y, float z );
    QVector3D& rotate(){ return _rotation; }

    // colors and materials
    void color( int r, int g, int b, int a = 255 );
    void color( QColor& c );
    QColor& color(){ return _color; }

    void specular( float s ){ /* TODO validate input */ _specular = s; }
    float specular(){ return _specular; }
    
    void reflectance( float r ){ /* TODO validate input */ _reflectance = r; }
    float reflectance(){ return _reflectance; }
    
    void emission( float e ){ /* TODO validate input */ _emission = e; }
    float emission(){ return _emission; }

    void metallic( bool m ){ _metallic = m; }
    bool metallic(){ return _metallic; }
    
    void drawObject();
    
protected:
    /**
     * Renders the object.
     */
    virtual void drawComponent() = 0;
    
private:
    bool _isVisible;
    
    QVector3D _translation;
    QVector3D _rotation;
    
    QColor _color;
    float _emission;
    float _specular;
    float _reflectance;
    bool _metallic;

    void updateMaterial();
};

#endif // _GFXOBJECT_H_
