#include "KeyState.h"

#include "Common.h"

#include "ModeFactory.h"
#include "MinorMode.h"

// singleton instance
KeyState* KeyState::_singleton = NULL;

KeyState* KeyState::INSTANCE()
{
    if( _singleton == NULL ) _singleton = new KeyState();

    return _singleton;
}

/**
 * Returns the key string.
 */
string KeyState::getKeyString( int key )
{
    string str;
    switch( key )
    {
        case 0:
            str += "C";
            break;
        case 1:
            str += "C#/Db";
            break;
        case 2:
            str += "D";
            break;
        case 3:
            str += "D#/Eb";
            break;
        case 4:
            str += "E";
            break;
        case 5:
            str += "F";
            break;
        case 6:
            str += "F#/Gb";
            break;
        case 7:
            str += "G";
            break;
        case 8:
            str += "G#/Ab";
            break;
        case 9:
            str += "A";
            break;
        case 10:
            str += "A#/Bb";
            break;
        case 11:
            str += "B";
            break;
    }
    
    return str;
}

/**
 * Constructor
 */
KeyState::KeyState()
:_key(0)
,_playMode(false)
//,_chordMode(CM_NOTE)
{
    mode( SMN_MAJOR );
}

/**
 * Destructor
 */
KeyState::~KeyState()
{
}

/**
 * Adds a listener
 */
void KeyState::addListener( KeyStateListener* l )
{
    _listeners.push_back( l );
}

/**
 * Returns the MIDI note value for the given scale number.
 * @param i the scale number. ranges from 1 to 8;
 * TODO: move to a util class.
 */
int KeyState::scaleToNote( unsigned int i, unsigned int o )
{
    return 12 * ( o + 1 ) + _key +  _mode->scaleToNote( i );
}

/**
 * Sets the key value.
 */
void KeyState::key( int i )
{ 
    assert( i > -1 && i < 12 ); 
    _key = i; 
    
    fireStateChanged();
}

/**
 * Sets the scale mode.
 */
void KeyState::mode( ScaleModeName smn )
{
    _mode = ModeFactory::INSTANCE()->getScaleMode( smn );
    
    fireStateChanged();
}

/**
 * Fires a state changed message
 */
void KeyState::fireStateChanged()
{
    //TODO: hand over change type
    for( unsigned int i = 0; i < _listeners.size(); i++ )
    {
        _listeners.at(i)->handleKeyStateChanged();
    }
}