#ifndef _MIDIENGINE_H_
#define _MIDIENGINE_H_

#include "Common.h"

#include <vector>

#include "RtMidi.h"

#include "MidiEvent.h"
#include "MidiEngineListener.h"

/**
 * The MidiEngine is the interface to RtMidi. The MidiEngine should only 
 * handle functions directly related to RtMidi.
 */
class MidiEngine
{
public:
	/**
	 * Default constructor
	 */
	MidiEngine();

	/**
	 * Destructor
	 */
	~MidiEngine();

	/**
	 * Initializes the MIDI engine.
	 */
	void initialize();

    /**
     * Adds a listener.
     */
    void addListener( MidiEngineListener* l );

    /**
     * Sends a Midi message.
     */
    void sendMidiMessage( MidiEvent& e );
	
private:
	/** Midi Input */
	RtMidiIn _rtMidiIn;

    /** Midi Output */
    RtMidiOut _rtMidiOut;

    /** Listener List */
    vector<MidiEngineListener*> _listeners;

	/** 
	 * Handles the callback 
	 */
	void handleCallback( double deltatime, vector< unsigned char > *message );

private:   
	/** 
	 * Callback wrapper 
	 */
	static void callback( double deltatime, vector< unsigned char > *message, 
                         void *userData );
	
};

#endif // _MIDIENGINE_H_
