#include "MidiEvent.h"

/**
 * Constructor
 * This constructor creates an all zero valued (empty) MidiEvent.
 * @param l - the number of bytes in the message.
 */
MidiEvent::MidiEvent(unsigned int l, double d)
:_deltaTime(d)
{
    if( l > 3 ) l = 3;
    
    for( unsigned int i = 0; i < l; i++ )
    {
        _message.push_back( 0 );
    }
}

/**
 * Constructor.
 */
MidiEvent::MidiEvent(int t, int n, double d)
:_deltaTime(d)
{
    if( t >= 0 ) _message.push_back( t );
    if( n >= 0 ) _message.push_back( n );
}

/**
 * Constructor.
 */
MidiEvent::MidiEvent(int t, int n, int v, double d)
:_deltaTime(d)
{
    if( t >= 0 ) _message.push_back( t );
    if( n >= 0 ) _message.push_back( n );
    if( v >= 0 ) _message.push_back( v );
}


/**
 * Copy constructor.
 */
MidiEvent::MidiEvent( const MidiEvent& m )
{
    for( unsigned int i = 0; i < m._message.size(); i++ )
    {
        _message.push_back( m._message[i] );
    }

    _deltaTime = m._deltaTime;
}

/**
 * Destructor.
 */
MidiEvent::~MidiEvent()
{
}

/**
 * Returns true if the message is a Note-On message.
 */
bool MidiEvent::isNoteOn()
{
    return ( type() == 144 && velocity() != 0 );
}

/**
 * Returns true if the message is a Note-On message.
 */
bool MidiEvent::isNoteOff()
{
    return( type() == 128 || ( type() == 144 && velocity() == 0 ) );
}

/**
 * Pretty prints the midi event.
 */
void MidiEvent::print()
{
    cout << "MidiEvent: [" ;

    for( unsigned int i = 0; i < _message.size(); i++ )
    {
        cout << (int)_message[i];
        if( i < _message.size() - 1 ) cout << ", ";
    }

    cout << "], delta = " << _deltaTime << endl;
}