#ifndef _MIDIEVENT_H_
#define _MIDIEVENT_H_

#include "Common.h"

#include <vector>

class MidiEvent
{
public:
    /**
     * Constructor
     * @param l - the number of bytes in the message.
     */
    MidiEvent(unsigned int l, double d);
    
	/**
	 * Constructor.
	 */    
    MidiEvent(int t, int n, double d);
        
	/**
	 * Constructor.
	 */
	MidiEvent( int t, int n, int v, double d );

    /**
     * Copy constructor.
     */
    MidiEvent( const MidiEvent& m );

	/**
	 * Destructor.
	 */
	~MidiEvent();

    bool isNoteOn();
    bool isNoteOff();

    // setters
	void type( int t ){ if( _message.size() > 0 ) _message[0] = t; }
	void note( int n ){ if( _message.size() > 1 ) _message[1] = n; }
	void velocity( int v ){ if( _message.size() > 2 ) _message[2] = v; }
	void deltaTime( double d ){ _deltaTime = d; }

    // getters
	int type(){ return _message.size() > 0 ? _message[0] : -1; }
	int note(){ return _message.size() > 1 ? _message[1] : -1; }
	int velocity(){ return _message.size() > 2 ? _message[2] : -1; }
	double deltaTime(){ return _deltaTime; }

    /** The number of bytes in the message */
    unsigned int length(){ return _message.size(); }

    /**
     * Pretty prints the event.
     */
    void print();

    vector<unsigned char>* toVector() { return &_message; }
	
private:
    vector<unsigned char> _message;
	double _deltaTime;
	
};

#endif // _MIDIEVENT_H_
