#include "Chord.h"

#include <algorithm>

/**
 * Default constructor
 */
Chord::Chord()
:_octave(4)
{
}

/**
 * Copy constructor
 */
Chord::Chord( Chord& c )
{
    copy( c );
}

/**
 * Destructor
 */
Chord::~Chord()
{
}

/**
 * Copies the chord.
 */
void Chord::copy( Chord& c )
{
    // copy octave
    _octave = c.octave();
    _label = c.label();

    // copy scale values
    _scaleValues.clear();

    unsigned int nc = c.noteCount();
    vector<int>* sv = c.scaleValues();

    for( unsigned int i = 0; i < nc; i++ )
    {
        _scaleValues.push_back( sv->at(i) );
    }
}

/**
 * Returns true if the chord contains the given chord.
 */
bool Chord::contains( Chord& c )
{
	if( c.noteCount() > noteCount() ) return false;

	vector<int>* sv = c.scaleValues();
	for( unsigned int i = 0; i < sv->size(); i++ )
	{
		if(  find( _scaleValues.begin(), _scaleValues.end(), sv->at(i) ) 
		   == sv->end() ) return false;
	}
	
	return true;
}

/**
 * Compares the scale values in the chords and
 * returns the number of common scale values.
 */
unsigned int Chord::compare( Chord& c )
{
    int count = 0;
    vector<int>* sv = c.scaleValues();

    for( unsigned int i = 0; i < noteCount(); i++ )
    {
        for( unsigned int j = 0; j < c.noteCount(); j++ )
        {
            if( _scaleValues.at(i) % 7 == sv->at(j) % 7 ) count++;
        }
    }
    
    return count;
}

/**
 * Clears the chord.
 */
void Chord::clear()
{
    _scaleValues.clear();
    _label.clear();
    _octave = 4;
}

/**
 * Adds a note with the given scale value.
 */
void Chord::addScaleValue( unsigned int i )
{
    assert( i > 0 );
	
    _scaleValues.push_back( i );

	// keep values sorted
	sort( _scaleValues.begin(), _scaleValues.end() );
}

/**
 * Sets the octave value.
 */
void Chord::octave( int i )
{ 
    assert( i > 0 ); 
    _octave = i; 
}

/**
 * Returns the root scale value.
 */
unsigned int Chord::root()
{
    // TODO: analyse chord to find the root note. (handle inversions)
    
    // assuming lowest note is the root(bass) note
    return ( _scaleValues.at(0) - 1 ) % 7 + 1;
}

/**
 * Returns the label.
 */
string& Chord::label( )
{ 
    return _label; 
}

/**
 * Pretty prints the content.
 */
void Chord::print()
{ 
	cout << "Chord(" << _label << "): [" ;

    for( unsigned int i = 0; i < _scaleValues.size(); i++ )
    {
        cout << (int)_scaleValues[i];
        if( i < _scaleValues.size() - 1 ) cout << ", ";
    }

    cout << "], octave = " << _octave << endl;
}