#include "ModeFactory.h"

#include "MajorMode.h"
#include "MinorMode.h"
#include "MinorHMode.h"
#include "LydianMode.h"

// singleton instance
ModeFactory* ModeFactory::_singleton = NULL;

ModeFactory* ModeFactory::INSTANCE()
{
    if( _singleton == NULL )
    {
        _singleton = new ModeFactory();
    }

    return _singleton;
}

/**
 * Constructor
 */
ModeFactory::ModeFactory()
{
    initialize();
}

/**
 * Destructor
 */
ModeFactory::~ModeFactory()
{
}

/**
 * Initializes the factory. The Scales are created here.
 */
void ModeFactory::initialize()
{
    _scales.push_back( new MajorMode() );
    _scales.push_back( new MinorMode() );
    _scales.push_back( new MinorHMode() );
    _scales.push_back( new LydianMode() );
}

/**
 * Returns the next scale mode name.
 */
ScaleModeName ModeFactory::nextMode( ScaleModeName s )
{
    return (ScaleModeName)( ( (unsigned int) s + 1 ) % (unsigned int) SMN_COUNT );
}

/**
 * Returns the next scale mode name.
 */
ScaleModeName ModeFactory::prevMode( ScaleModeName s )
{
//    unsigned int toInt = s;
    if( s == 0 ) return (ScaleModeName)( (unsigned int)SMN_COUNT - 1 );
    else return (ScaleModeName)( (unsigned int)s - 1 );
//    return (ScaleModeName)( ( (unsigned int) s + 1 ) % (unsigned int) SMN_COUNT );
} 

/**
 * Returns the scale mode.
 */
ScaleMode* ModeFactory::getScaleMode( ScaleModeName s )
{
    return _scales[s];
}