#ifndef _SCALEMODE_H_
#define _SCALEMODE_H_

#include "Common.h"
#include <vector>

enum ScaleModeName
{
    SMN_MAJOR = 0,
    SMN_MINOR,
    SMN_MINOR_H,
    SMN_LYDIAN,
    SMN_COUNT
};

/**
 * Diatonic scale modes. 
 * @see http://en.wikipedia.org/wiki/Gregorian_modes
 *
 * Subclasses must set the scales in createScale().
 * This class can be extended to non-diatonic scales, 
 * e.g. chromatic, pentatonic.
 * Currently the uses and implementations assume
 * a diatonic scale.
 * The 8th or the upper tonic is also added.
 */
class ScaleMode
{
public:
	ScaleMode( ScaleModeName name, const char* s );
	virtual ~ScaleMode();

    ScaleModeName scaleModeName(){ return _name; }

    int scaleToNote( unsigned int i );

    string& toString(){ return _nameString; }

protected:
    ScaleModeName _name;
    virtual void createScale() = 0;

    string _nameString;
    
	vector<int> _offsets; // extendable to different scales.
};

#endif // _SCALEMODE_H_
