#include "PadController.h"

#include "KeyState.h"

/**
 * Constructor
 */
PadController::PadController( MidiEngine& m )
:_midiRemapper(m)
{
	// intialize MIDI remapper
    _midiRemapper.initialize();
    m.addListener( &_midiRemapper );
    _midiRemapper.addPadEventListener( &_pcObject );
}

/**
 * Destructor
 */
PadController::~PadController()
{
}

/**
 * Adds a chord to the pads.
 */
bool PadController::addChord( Chord& c )
{
    unsigned int id = 0;
    bool result = _midiRemapper.addChord( c, &id );
    if( result )
    {
        PadData& pd = _midiRemapper.padData( id );

        if( c.noteCount() < 1 )
        {
            string empty( "empty" );
            _pcObject.pad( pd.id() ).setText( empty );
            _pcObject.pad( pd.id() ).setOctave( 0 );
        }
        else
        {
            _pcObject.pad( pd.id() ).setText( c.label() );
            _pcObject.pad( pd.id() ).setOctave( c.octave() );
        }
    }

    return result;
}

/**
 * Clears the pad of the given id.
 */
void PadController::clearPad( unsigned int i )
{
    _pcObject.clearPad( i );
    _midiRemapper.padData( i ).clear();
}

/**
 * Clears all pad data.
 */
void PadController::clearAll()
{
    _midiRemapper.clearAllData();
    _pcObject.clearPads();
}

/**
 * Updates the UI with the PadData in the MidiRemapper
 */
void PadController::updateUI()
{
    for( unsigned int i = 0; i < _midiRemapper.numPads(); i++ )
    {
        PadTextObject& pad = _pcObject.pad( i );
        Chord& chord = _midiRemapper.padData( i ).chord();

        if( chord.noteCount() < 1 )
        {
            string empty( "empty" );
            pad.setText( empty );
            pad.setOctave( 0 );
        }
        else
        {
            pad.setText( chord.label() );
            pad.setOctave( chord.octave() );
        }
    }
}

// @Override
void PadController::handleKeyStateChanged()
{
    _midiRemapper.updatePadChords();
}