#include "PadControllerObject.h"

/**
 * Constructor
 */
PadControllerObject::PadControllerObject()
{
   initialize();
}

/**
 * Destructor
 */
PadControllerObject::~PadControllerObject()
{
}

/**
 * Initializes the components.
 */
void PadControllerObject::initialize()
{
    float ps = 20; // pad size
    float sp = 5; // spacing
    
    float w = ps * 4 + sp * 7;
    _body.width( w );
    _body.height( w );
    _body.depth( 20 );
    _body.translate( -w/2, -w/2, -20 );
    _body.color( 192, 192, 192 );
    _body.specular( 1 );
    _body.reflectance( 0.3 );
    _objects.push_back( &_body );

    w = ps * 4 + sp * 5; //11;
    _padRim.width( w );
    _padRim.height( w );
    _padRim.depth( 10 );
    _padRim.translate( -w/2, -w/2, -8.5f );
    _padRim.color( 32, 32, 32 );
    _padRim.specular( 1 );
    _padRim.reflectance( 0 );
    _objects.push_back( &_padRim );
    
    int r = 0;
    int c = 0;
    w = ps;
    for( int i = 0; i < 16; i++ )
    {
        r = i % 4;
        c = i / 4;

        _pads[i].padObject().width( w );
        _pads[i].padObject().height( w );
        _pads[i].padObject().depth( 3 );
        _pads[i].translate( -w / 2 + ( ps + sp ) * ( r - 1.5 ), 
                           -w / 2 + ( ps + sp ) * ( c - 1.5 ), 
                           0 );
        _pads[i].padObject().color( PadObject::IDLE_COLOR );
        _pads[i].padObject().emission( 0.5f );
        _pads[i].padObject().specular( 10 );
        _pads[i].padObject().reflectance( 0 );
        _objects.push_back( &_pads[i] );
    }
}

/**
 * Clears pad with the given id.
 */
void PadControllerObject::clearPad( unsigned int i )
{
    assert( i < NUM_PADS );
    
    _pads[i].setText("empty");
    _pads[i].setOctave( 0 );
}

/**
 * Clears all the pads
 */
void PadControllerObject::clearPads()
{
    for( int i = 0; i < 16; i++ )
    {
        clearPad( i );
        // _pads[i].setText("empty");
        // _pads[i].setOctave( 0 );
    }
}

// @Override
void PadControllerObject::drawComponent()
{    
    for( unsigned int i = 0; i < _objects.size(); i++ )
    {
        _objects[i]->drawObject();
    }
}

// @Override
void PadControllerObject::handlePadEvent( PadEvent& e )
{
    PadObject& pad = _pads[e.padId()].padObject();
    
    if( e.midiEvent().isNoteOn() ) 
    { // note on
        float v = e.midiEvent().velocity() / 127.0;
        pad.hit( v );
    }
    else if( e.midiEvent().isNoteOff() )
    { // note off
        pad.idle();
    }
}
