#include "PadObject.h"

#include "CommonUtil.h"

QColor PadObject::IDLE_COLOR = QColor( 192, 192, 192 );
QColor PadObject::HIT_COLOR = QColor( 127, 0, 0 );
QColor PadObject::SUGGEST_COLOR = QColor( 0, 127, 0 );

/**
 * Constructor
 */
PadObject::PadObject( float width, float height, float depth )
:GfxBoxObject(width, depth, height)
,_padState(PS_IDLE)
,_mix(0)
,_suggest(0)
,_period(2)
,_lb(0.5f)
,_ub(1)
{
}

/**
 * Destructor
 */
PadObject::~PadObject()
{
}

/**
 * Sets color to idle color. 
 */
void PadObject::idle()
{
	_padState = PS_IDLE;
    _suggest = 0;
//	color( IDLE_COLOR );
}

/**
 * Sets color to hit color. 
 */
void PadObject::hit( float v )
{
	_padState = PS_HIT;

    gettimeofday( &_then, NULL );

    float min = 0.5f;
    float nv = (1-min)*v + min;
    
    QColor mc = CommonUtil::mixColors( IDLE_COLOR, HIT_COLOR, nv );
    color( mc );
}

/**
 * Sets color to suggest color. 
 */
void PadObject::suggest( float v )
{
    if( _padState == PS_HIT ) return; // if its hit then don't suggest.
    if( _suggest > v ) return; // if it has a stronger suggestion, leave.
    
	_padState = PS_SUGGEST;

    _mix = 1;

    QColor mc = CommonUtil::mixColors( IDLE_COLOR, SUGGEST_COLOR, v );
    color( mc );
}

// @Override
void PadObject::color( int r, int g, int b, int a )
{
    // TODO: validate values
    _currentColor.setRgb( r, g, b, a );
}

// @Override
void PadObject::color( QColor& c )
{
    _currentColor = c;
}

// @Override
void PadObject::drawComponent()
{	
	// pulsate color    
	if( _padState == PS_HIT )
	{
        struct timeval now;
        gettimeofday( &now, NULL );

        float t = ( now.tv_sec - _then.tv_sec ) 
            + ( now.tv_usec - _then.tv_usec ) / 1000000.0;
        
		_mix = ( 1 + ::cos( 2 * PI * (1.0/_period) * t ) ) 
			* ( _ub - _lb ) / 2 + _lb;

        QColor mc = CommonUtil::mixColors( IDLE_COLOR, _currentColor, _mix);
        GfxObject::color( mc );
	} 
	else if( _padState == PS_SUGGEST )
    {
        GfxObject::color( _currentColor );
    }
	else // PS_IDLE
    {
        if( _mix != 0 ) // fade out
        {
            _mix -= 0.1f;
            if( _mix <= 0 ) 
            {
                _mix = 0;
                color( IDLE_COLOR );
            }

            QColor mc = CommonUtil::mixColors( IDLE_COLOR, _currentColor, 
                                              _mix);
            GfxObject::color( mc );
        }
        else
        {
            GfxObject::color( IDLE_COLOR );
        }
    }

	// draw
	GfxBoxObject::drawComponent();
    _textObject.drawObject(); 
    // calling drawObject of the after drawing the box object 
    // may cause translation related problems...
}