#include "PadData.h"

#include "Common.h"

#include "KeyState.h"

/**
 * Constructor
 */
PadData::PadData( unsigned int id, unsigned int b )
:_id(id)
,_bank(b)
{
}

/**
 * Copy constructor
 */
PadData::PadData( const PadData& p )
{
    _id = p._id;
    _bank =  p._bank;
    _inputNote = p._inputNote;

    for( unsigned int i = 0; i < p._outputNotes.size(); i++ )
    {
        _outputNotes.push_back( p._outputNotes[i] );
    }
}

/**
 * Destructor
 */
PadData::~PadData()
{
}

/**
 * Sets the pad id.
 */
void PadData::id( unsigned int id )
{
    _id = id;
}

/**
 * Sets the input note value. A Midi input with note value v will be remapped
 * according to this PadData.
 */
void PadData::inputNote( int v )
{
    if( v < 0 ) throw ClassException( "PadData", "inputNote", "invalid value" );

    _inputNote = v;    
}

/**
 * Sets the chord
 */
void PadData::setChord( Chord& c )
{
    // copy chord
    _chord.copy( c );

    update();
}

/**
 * Updates the output notes.
 */
void PadData::update()
{
    // pre compute notes
	clearOutputNotes();
	
    KeyState* ks = KeyState::INSTANCE();
    vector<int>* notes = _chord.scaleValues();
    unsigned int o = _chord.octave();
    
    for( unsigned int i = 0; i < notes->size(); i++ )
    {
        if( ks->mode() == SMN_MINOR_H 
           && notes->at(0) == 3
           && notes->at(i) == 7 )
        { // lower note
            addOutputNote( ks->scaleToNote( notes->at( i ), o ) - 1 );
        }
        else addOutputNote( ks->scaleToNote( notes->at( i ), o ) );
    }
}

/**
 * Clears all data.
 */
void PadData::clear()
{
    _chord.clear();
    update();
}

/**
 * Adds an output note value.
 */
void PadData::addOutputNote( int v )
{
    if( v < 0 ) throw ClassException( "PadData", "inputNote", "invalid value" );

    _outputNotes.push_back( v );
}


/**
 * Clears all output notes.
 */
void PadData::clearOutputNotes()
{
    _outputNotes.clear();
}