#include "ChordItem.h"

#include "CommonUtil.h"

int ChordItem::ITEM_WIDTH = 20;
int ChordItem::ITEM_HEIGHT = 10;

QColor ChordItem::IDLE_COLOR = QColor( 160, 160, 160 );
QColor ChordItem::SELECT_COLOR = QColor( 255, 255, 255 );
QColor ChordItem::SUGGEST_COLOR = QColor( 0, 127, 0 );

/**
 * Constructor
 */
ChordItem::ChordItem()
:_boxObject(ITEM_WIDTH, ITEM_HEIGHT, 3)
,_isSelected(false)
,_isSuggested(false)
{
    initialize();
}

/**
 * Destructor
 */
ChordItem::~ChordItem()
{
}

/**
 * Sets the text
 */
void ChordItem::setText( string& s )
{
    _textObject.setText( s );
}

/**
 * Sets the item idle.
 */
void ChordItem::idle()
{
    _isSelected = false;
    
    if( !_isSuggested )
    {
        _boxObject.color( IDLE_COLOR );
        _boxObject.emission(0.5f);
    }
}

/**
 * Selects the item
 */
void ChordItem::select()
{
    _isSelected = true;

    if( !_isSuggested )
    {
        _boxObject.color( SELECT_COLOR );
        _boxObject.emission(1);
    }
}

/**
 * Suggests the item
 */
void ChordItem::suggest( float v )
{
    QColor& currentColor = _isSelected ? SELECT_COLOR : IDLE_COLOR;

    if( v == 0 )
    {
        _isSuggested = false;
        _boxObject.color( currentColor );
        _boxObject.emission( _isSelected ? 1 : 0.5f );
    }
    else
    {
        _isSuggested = true;
        QColor mc = CommonUtil::mixColors( currentColor, SUGGEST_COLOR, v );
        _boxObject.color( mc );
        _boxObject.emission( 0.5f );
    }
}

/**
 * Initializes the components
 */
void ChordItem::initialize()
{
    string t = "text";
    _textObject.setFontSize( 5 );
    _textObject.setText( t );
    _textObject.translate( 1, 1, 3 );
    _textObject.emission( 1 );
    _textObject.color( 16, 16, 16 );

    _boxObject.color( IDLE_COLOR );
    _boxObject.emission( 0.5f );
    _boxObject.specular( 10 );
    _boxObject.reflectance( 0 );
}

// @Override
void ChordItem::drawComponent()
{
    _textObject.drawObject();
    _boxObject.drawObject();
}