#include "PadStatBoxPlot.h"

#include "CommonUtil.h"

QColor PadStatBoxPlot::MIN_COLOR(16, 16, 16);
QColor PadStatBoxPlot::MAX_COLOR(255, 255, 255);

/**
 * Constructor
 */
PadStatBoxPlot::PadStatBoxPlot( PadStatistics& ps )
:_padStats(ps)
,_enableDepth(true)
,_xInt(0, 0, 0.3f)
,_xPos(0)
,_outPos(-50)
{
    initialize();
    _padStats.addListener( this );
}

/**
 * Destructor
 */
PadStatBoxPlot::~PadStatBoxPlot()
{
}

// This method will break down if the the x position is changed via
// gfxObject.translate().setX(x);
// @Override
void PadStatBoxPlot::translate( float x, float y, float z )
{
    _xPos = x; // save the assigned position
    
    GfxObject::translate( x, y, z );
}

/**
 * Shows the chord selector.
 * TODO: a layout object should handle this.
 */
void PadStatBoxPlot::show()
{
    setVisible( true );
    _xInt.goal( 0 );
}

/**
 * Hides the chord selector.
 * TODO: a layout object should handle this.
 */
void PadStatBoxPlot::hide()
{
    _xInt.goal( _outPos );
}

/**
 * Enables the depth to show the number of occurences.
 */
void PadStatBoxPlot::enableDepth( bool b )
{
    _enableDepth = b;

    updateData();
}

int boxSize = 4;

/**
 * Initializes the components
 */
void PadStatBoxPlot::initialize()
{
    unsigned int m = 0;
    unsigned int n = 0;
    for( unsigned int i = 0; i < 49; i++ )
    {
        m = i % 7;
        n = i / 7;
        
        _boxObjects[i].width( boxSize );
        _boxObjects[i].height( boxSize );
        _boxObjects[i].translate( m * ( boxSize + 0.5f ), 
                                 (6 - n) * ( boxSize + 0.5f ), 
                                 0 ); // flip y to match matrix form
        _boxObjects[i].color( MIN_COLOR );
    }
}

unsigned int DEPTH = 10;

/**
 * Pulls the data from the PadStatistics object and updates the plot.
 */
void PadStatBoxPlot::updateData()
{
    unsigned int* data = _padStats.countArray();

    float mix = 0;
    unsigned int col[7];
    unsigned int idx[7];

    // find maximum
    unsigned int max = 0;
    for( unsigned int n = 0; n < 49; n++ )
    {
        if( data[n] > max ) max = data[n];
    }
    max = max == 0 ? 1 : max;

    for( unsigned int j = 0; j < 7; j++ )
    {// for each column
        memset( col, 0, 7 * sizeof(unsigned int) );

        // sort column
        for( unsigned int i = 0; i < 7; i++ ) col[i] = data[i * 7 + j];
        CommonUtil::sortData( col, idx, 7 );

        // color object by order
        // sort column
        for( unsigned int i = 0; i < 7; i++ )
        {
            mix = ( col[i] == 0 ) ? 0 : ( idx[i] / 6.0 );

            QColor mc = CommonUtil::mixColors( MIN_COLOR, MAX_COLOR, mix );
            _boxObjects[i * 7 + j].color( mc );

            // calculate depth
            if( _enableDepth ) _boxObjects[i * 7 + j].depth(
                                    ( col[i] + 1 ) / (float)max * DEPTH );
            else _boxObjects[i * 7 + j].depth( 1 );
        }
    }
}

// @Override
void PadStatBoxPlot::handleChange()
{
    updateData();
}

// @Override
void PadStatBoxPlot::drawComponent()
{
    for( unsigned int i = 0; i < 49; i++ )
    {
        _boxObjects[i].drawObject();
    }

    _xInt.interpolate();
    GfxObject::translate().setX( _xPos + _xInt.value() );
    if( ::abs(_xInt.value() - _outPos) < 1  ) 
    {
        if( isVisible() ) setVisible( false );
    }
}