#include "PadStatistics.h"

#include "KeyState.h"

/**
 * Constructor
 */
PadStatistics::PadStatistics()
:_prevRoot(10) // use any number above 7 to indicate start
{
    memset( _countArray, 0, 49*sizeof(unsigned int) );
    memset( _columnSum, 0, 7*sizeof(unsigned int) );
}

/**
 * Destructor
 */
PadStatistics::~PadStatistics()
{
}

/**
 * Resets the statistics.
 */
void PadStatistics::reset()
{
    memset( _countArray, 0, 49*sizeof(unsigned int) );
    memset( _columnSum, 0, 7*sizeof(unsigned int) );
    _prevRoot = 10; // use any number above 7 to indicate start
    
    fireEvent();
}

/**
 * Pretty prints the stats array.
 */
void PadStatistics::printArray()
{
    cout << "PadStatistics" << endl;
    for( unsigned int i = 0; i < 7; i++ )
    {
        for( unsigned int j = 0; j < 7; j++ )
        {
            cout << _countArray[i * 7 + j];
            
            if( j < 6 ) cout << "\t";
        }
        cout << endl;
    }
}

/**
 * Fires an event to the listeners.
 */
void PadStatistics::fireEvent()
{
    for( unsigned int i = 0; i < _listeners.size(); i++ )
    {
        _listeners.at(i)->handleChange();
    }
}

// @Override
void PadStatistics::handlePadEvent( PadEvent& e )
{
    // only save results in playmode
    if( !KeyState::INSTANCE()->isPlayMode() ) return; 
    
    if( !e.midiEvent().isNoteOn() ) return;
    
    Chord& c = e.padData().chord();
    if( c.noteCount() == 0 ) return; // empty pad
    
    // assuming lowest note is the root(bass) note
    unsigned int root = c.root() - 1;//( c.scaleValues()->at(0) - 1 ) % 7;

    if( _prevRoot < 7 ) 
    {
        _columnSum[_prevRoot]++;
        _countArray[root * 7 + _prevRoot]++;
    }
    
    _prevRoot = root;

    fireEvent();
}

// @Override
vector<unsigned int>& PadStatistics::getSuggestion( unsigned int i )
{
    _suggestList.clear();
    for( unsigned int n = 0; n < 7; n++ )
    {
        _suggestList.push_back( _countArray[n * 7 + i] );
    }
    
    return _suggestList;
}