#include "TextSuggestor.h"

#include "CommonUtil.h"

unsigned int TextSuggestor::SPACING = 10;

/**
 * Constructor
 */
TextSuggestor::TextSuggestor( PadStatistics& ps )
:_padStats( ps )
{
    _padStats.addListener( this );
}

/**
 * Destructor
 */
TextSuggestor::~TextSuggestor()
{
}

// @Override
void TextSuggestor::handleChange()
{
    unsigned int root = _padStats.root();

    if( root > 6 ) return; // TODO: handle reset

    // create text object
    string l;
    l.append( CommonUtil::numToRoman( root + 1 ) );
    _textObjects.push_back( new GfxFtStringObject( l ) );
    Interpolator xPos( _textObjects.size() * SPACING, 
                      _textObjects.size() * SPACING, 0.3 );
    _xPosList.push_back( xPos );

    l.clear();
    l.append("-");
    _textObjects.push_back( new GfxFtStringObject( l ) );
    Interpolator spacePos( _textObjects.size() * SPACING, 
                          _textObjects.size() * SPACING, 0.3 );
    _xPosList.push_back( spacePos );

    // move objects
    if( _textObjects.size() > 2 * 5 )
    {
        // delete front
        _textObjects.pop_front();
        _textObjects.pop_front();
        _xPosList.pop_front();
        _xPosList.pop_front();

        for( unsigned int i = 0; i < _xPosList.size(); i++ )
        {
            _xPosList.at( i ).goal( i * SPACING );
        }
    }
}

// @Override
void TextSuggestor::drawComponent()
{
    // interpolate positions
    for( unsigned int i = 0; i < _xPosList.size(); i++ )
    { 
        _xPosList.at( i ).interpolate();

        // linear interpolate
/*        float v = _xPosList.at( i ).value();
        if( v >= _xPosList.at( i ).goal() ) 
        {
            v = ( v - 2 < _xPosList.at( i ).goal() ) ? 
                _xPosList.at( i ).goal() : v - 2;
            _xPosList.at( i ).value( v );
        }*/
    }

    // draw objects
    GfxFtStringObject* to;
    for( unsigned int i = 0; i < _textObjects.size(); i++ )
    {
        to = _textObjects.at( i );
        to->translate().setX( - _textObjects.at( i )->width() / 2 
                             + _xPosList.at( i ).value() );
        to->drawObject();
    }
}