#include "TonalMap.h"

/**
 * Constructor
 */
TonalMap::TonalMap()
:_suggestionMap(7)
{
    initialize();
}

/**
 * Destructor
 */
TonalMap::~TonalMap()
{
}

/**
 * Returns a suggestion vector
 */
vector<unsigned int>& TonalMap::getSuggestion( unsigned int i )
{
    assert( i < _suggestionMap.size() );

    return _suggestionMap.at( i );
}

/**
 * Initializes suggestion map.
 * Current tonal mapping from
 * "Tonal Harmony", by Kostka and Payne
 */
void TonalMap::initialize()
{
    // I: Tonic
    unsigned int i = 0;
    _suggestionMap.at( i ).push_back( i+1 ); // self
    _suggestionMap.at( i ).push_back( 3 ); // tonic
    _suggestionMap.at( i ).push_back( 4 ); // sub-dominants
    _suggestionMap.at( i ).push_back( 6 );
    _suggestionMap.at( i ).push_back( 2 );
    _suggestionMap.at( i ).push_back( 5 ); // dominants

    // II: Super-tonic (sub-dominant)
    i++;
    _suggestionMap.at( i ).push_back( i+1 ); // self
    _suggestionMap.at( i ).push_back( 5 ); // dominants
    _suggestionMap.at( i ).push_back( 7 );

    // III: Mediant (tonic)
    i++;
    _suggestionMap.at( i ).push_back( i+1 ); // self
    _suggestionMap.at( i ).push_back( 4 );
    _suggestionMap.at( i ).push_back( 6 );

    // IV: Sub-dominant
    i++;
    _suggestionMap.at( i ).push_back( i+1 ); // self 
    _suggestionMap.at( i ).push_back( 1 ); // tonic
    _suggestionMap.at( i ).push_back( 2 ); // sub-dominants
    _suggestionMap.at( i ).push_back( 5 ); // dominants
    _suggestionMap.at( i ).push_back( 7 );

    // V: Dominant
    i++;
    _suggestionMap.at( i ).push_back( i+1 ); // self
    _suggestionMap.at( i ).push_back( 6 ); // tonic
    _suggestionMap.at( i ).push_back( 1 ); 

    // VI: Sub-mediant (tonic)
    i++;
    _suggestionMap.at( i ).push_back( i+1 ); // self
    _suggestionMap.at( i ).push_back( 4 ); // sub-dominants
    _suggestionMap.at( i ).push_back( 2 );

    // VII: Sub-tonic/Leading (dominant)
    i++;
    _suggestionMap.at( i ).push_back( i+1 ); // self
    _suggestionMap.at( i ).push_back( 1 ); // tonic
}