#include "CommonUtil.h"

#include <vector>
#include <algorithm>

/**
 * Converts numbers to roman numerals.
 * NOTE: only handles numbers up to 10.
 */
string CommonUtil::numToRoman( unsigned int i, bool lower )
{
    string roman;
    switch( i )
    {
        case 0:
            roman.append( lower ? "o" : "O" );
            break;
        case 1:
            roman.append( lower ? "i" : "I" );
            break;
        case 2:
            roman.append( lower ? "ii" : "II" );
            break;
        case 3:
            roman.append( lower ? "iii" : "III" );
            break;
        case 4:
            roman.append( lower ? "iv" : "IV" );
            break;
        case 5:
            roman.append( lower ? "v" : "V" );
            break;
        case 6:
            roman.append( lower ? "vi" : "VI" );
            break;
        case 7:
            roman.append( lower ? "vii" : "VII" );
            break;
        case 8:
            roman.append( lower ? "viii" : "VIII" );
            break;
        case 9:
            roman.append( lower ? "ix" : "IX" );
            break;
        case 10:
            roman.append( lower ? "x" : "X" );
            break;
        default:
            roman.append( "Out-of-bounds" );
            break;
    }
    
    return roman;
}

/**
 * Mixes the two colors. This does not mix the alpha channels.
 */
QColor CommonUtil::mixColors( QColor c1, QColor c2, float m )
{
    if( m > 1 ) m = 1;
    if( m < 0 ) m = 0;
    
    return QColor( ( 1 - m ) * c1.red() + m * c2.red(), 
                 ( 1 - m ) * c1.green() + m * c2.green(), 
                 ( 1 - m ) * c1.blue() + m * c2.blue() );
}

class IndexedValue
{
public:
    IndexedValue( unsigned int v, unsigned int i ):_v(v),_i(i){}
    IndexedValue( const IndexedValue& iv ){ _v = iv._v; _i = iv._i; }
    
    unsigned int _v; // value
    unsigned int _i; // index
};

bool compareValues( IndexedValue i, IndexedValue j) 
{ 
    return ( i._v < j._v );
}

/**
 * Returns the indices in sorted order.
 * TODO: optimize by using a O( n log(n) ) sorting algorithm
 */
void CommonUtil::sortData( unsigned int* d, unsigned int* idx, 
                          unsigned int n )
{
    vector<IndexedValue> data;

    for( unsigned int i = 0; i < n; i++ )
    {
        data.push_back( IndexedValue( d[i], i ) );
    }

    sort( data.begin(), data.end(), compareValues );

//    cout << "Sorted index: ";
    for( unsigned int i = 0; i < n; i++ )
    {
        idx[data.at(i)._i] = i;
//        cout << idx[i];
    }
//    cout << endl;
}












